// Copyright © 2010-2020 Singular Inc. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "SingularLinkParams.h"
#include "SingularShortLinkParams.h"
#include "SingularConversionValuesParams.h"
#include "SingularDeviceAttributionCallbackHandler.h"
#include "SingularDelegates.generated.h"

// Dynamic delegates
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FDeviceAttributionCallbackHandler, const FSingularDeviceAttributionCallbackHandler& , deviceAttributionCallbackHandler);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnConversionValueUpdated, int, conversionValue);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnConversionValuesUpdated, const FSingularConversionValuesParams&, conversionValuesParams);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FSingularLinksHandler, const FSingularLinkParams&, LinkParams);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FSingularShortLinksHandler, const FSingularShortLinkParams&, LinkParams);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FSingularSdidReceivedHandler, const FString, result);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FSingularDidSetSdidHandler, const FString, result);

UCLASS( ClassGroup=(SingularSDK), meta=(BlueprintSpawnableComponent) )
class SINGULARSDK_API USingularDelegates : public UActorComponent
{
    GENERATED_BODY()
    
public:
    USingularDelegates(const FObjectInitializer& ObjectInitializer);

    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FOnConversionValueUpdated OnConversionValueUpdated;

    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FOnConversionValuesUpdated OnConversionValuesUpdated;

    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FSingularLinksHandler OnSingularLinksResolved;

    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FSingularShortLinksHandler OnSingularShortLinksResolved;
    
    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FDeviceAttributionCallbackHandler DeviceAttributionCallbackHandler;
    
    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FSingularSdidReceivedHandler OnSingularSdidReceived;
    
    UPROPERTY(BlueprintAssignable, Category = "Singular-SDK")
    FSingularDidSetSdidHandler OnSingularDidSetSdid;
};
